@@ -2,6 +2,7 @@ from datetime import datetime, timedelta
 import aiohttp
 import asyncio
 from typing import Union
+from .regions import APIRegion
 
 class API():
     """This is our core API class with methods for interacting with Battle.net's various APIs
@@ -10,48 +11,21 @@ class API():
     :type client_id: str
     :param client_secret: A Battle.net Project Client Secret - Generated at https://develop.battle.net/access/clients
     :type client_secret: str
-    :param client_region: The Battle.net/WoW API region which we submit requests to, defaults to 'us'
-    :type client_region: str, optional
-    :param client_locale: The locale used for API requests. Compatbility varies by region, see https://develop.battle.net/documentation/guides/regionality-and-apis, defaults to 'en_US'
-    :type client_locale: str, optional
+    :param client_region: The Battle.net/WoW API region which we submit requests to, defaults to APIRegion.US
+    :type client_region: str, APIRegion, optional
     :param max_parallel_requests: The maximum number of parallel aiohttp requests
     :type max_parallel_requests: int, optional
     :param max_request_retries: The maximum number of aiohttp request retries (min 1)
     :type max_request_retries: int, optional
     :param request_retry_delay: The delay between aiohttp request retries (min 0) (This value is in seconds)
     :type request_retry_delay: int, optional
+    :param request_debugging: Whether exceptions should be raised requests resulting in an HTTP error (if set to false requests resulting in an HTTP error will simply return None instead of raising an exception)
+    :type request_debugging: bool, optional
     """
 
-    def __init__(self, client_id:str, client_secret:str, client_region:str='us', client_locale:str='en_US', max_parallel_requests:int=50, max_request_retries:int=3, request_retry_delay:int=1):
+    def __init__(self, client_id:str, client_secret:str, client_region:Union[APIRegion, str], max_parallel_requests:int=50, max_request_retries:int=3, request_retry_delay:int=1, request_debugging:bool=True):
         """Constructor method
         """
-        self.api_regions = {
-            'us': {
-                'oauth_api_hostname': 'https://us.battle.net{api_endpoint}',
-                'game_api_hostname': 'https://us.api.blizzard.com{api_endpoint}',
-                'supported_locales': ['en_US', 'es_MX', 'pt_BR']
-            },
-            'eu': {
-                'oauth_api_hostname': 'https://eu.battle.net{api_endpoint}',
-                'game_api_hostname': 'https://eu.api.blizzard.com{api_endpoint}',
-                'supported_locales': ['en_GB', 'es_ES', 'fr_FR', 'ru_RU', 'de_DE', 'pt_PT', 'it_IT']
-            },
-            'kr': {
-                'oauth_api_hostname': 'https://kr.battle.net{api_endpoint}',
-                'game_api_hostname': 'https://kr.api.blizzard.com{api_endpoint}',
-                'supported_locales': ['ko_KR']
-            },
-            'tw': {
-                'oauth_api_hostname': 'https://tw.battle.net{api_endpoint}',
-                'game_api_hostname': 'https://tw.api.blizzard.com{api_endpoint}',
-                'supported_locales': ['zh_TW']
-            },
-            'cn': {
-                'oauth_api_hostname': 'https://www.battlenet.com.cn{api_endpoint}',
-                'game_api_hostname': 'https://gateway.battlenet.com.cn{api_endpoint}',
-                'supported_locales': ['zh_CN']
-            }
-        }
 
         self.client_id = client_id
         self.client_secret = client_secret
@@ -59,7 +33,6 @@ class API():
         self.client_locale = None
 
         self.setRegion(client_region)
-        self.setLocale(client_locale)
 
         self.access_tokens = {}
         
@@ -69,16 +42,18 @@ class API():
         
         self.request_retry_delay = request_retry_delay if request_retry_delay > 0 else 1
         
+        self.request_debugging = request_debugging
+        
 
-    async def getRegion(self) -> str:
+    async def getRegion(self) -> APIRegion:
         """Returns the current region being used for API requests
 
         :return: The current region being used for API requests
         :rtype: str
         """
-        return self.client_region
+        return self.client_region.name
 
-    async def getLocale(self) -> str:
+    async def getLocale(self) -> APIRegion:
         """Returns the current locale being used for API requests
 
         :return: The current locale being used for API requests
@@ -86,19 +61,28 @@ class API():
         """
         return self.client_locale
 
-    def setRegion(self, region:str) -> None:
+    def setRegion(self, region:Union[APIRegion, str]) -> None:
         """Sets the region we'll use for API requests
 
         :param region: The desired region to be used for API requests
-        :type region: str
-        :raises InvalidRegionException: Raised when the provided region isn't in our supported regions dictionary
+        :type region: str, APIRegion
+        :raises InvalidRegionException: Raised when the provided region isn't supported/found
         """
-        if region in self.api_regions:
+        if region in list(APIRegion):
             self.client_region = region
-            self.client_locale = self.api_regions[region]['supported_locales'][0]
+            self.client_locale = region.value['supported_locales'][0]
+            return
+        elif isinstance(region, str):
+            for i in list(APIRegion):
+                if i.name.lower() == region.lower():
+                    self.client_region = i
+                    self.client_locale = i.value['supported_locales'][0]
+                    return
+            raise InvalidRegionException('Invalid API Region {}, supported regions are {}'.format(
+                region, list(APIRegion.__members__.keys())))
         else:
             raise InvalidRegionException('Invalid API Region {}, supported regions are {}'.format(
-                region, list(self.api_regions.keys())))
+                region, list(APIRegion.__members__.keys())))
 
     def setLocale(self, locale:str) -> None:
         """Sets the locale we'll use for API requests
@@ -107,11 +91,11 @@ class API():
         :type locale: str
         :raises InvalidLocaleException: Raised when the string given doesn't match any of the current region's supported locales
         """
-        if locale and locale in self.api_regions[self.client_region]['supported_locales']:
+        if locale and locale in self.client_region.value['supported_locales']:
             self.client_locale = locale
         else:
             raise InvalidLocaleException('Invalid Regional Locale {}, supported locales for {} are {}'.format(
-                locale, self.client_region, self.api_regions[self.client_region]['supported_locales']))
+                locale, self.client_region.name, self.client_region.value['supported_locales']))
 
     async def getHostname(self) -> str:
         """Returns the current region's hostname for Game API requests
@@ -119,7 +103,7 @@ class API():
         :return: The current region's hostname for Game API requests
         :rtype: str
         """
-        return self.api_regions[self.client_region]['game_api_hostname']
+        return self.client_region.value['game_api_hostname']
     
     async def getOAuthHostname(self) -> str:
         """Returns the current region's hostname for OAuth API requests
@@ -127,7 +111,7 @@ class API():
         :return: The current region's hostname for OAuth API requests
         :rtype: str
         """
-        return self.api_regions[self.client_region]['oauth_api_hostname']
+        return self.client_region.value['oauth_api_hostname']
     
     async def getAccessToken(self) -> str:
         """Returns and / or generates an API access token using the provided credentials in the class constructor
@@ -137,8 +121,8 @@ class API():
         :rtype: str
         """
 
-        if self.access_tokens and self.access_tokens[self.client_region] and self.access_tokens[self.client_region]['Expires'] > datetime.now():
-            return self.access_tokens[self.client_region]['Token']        
+        if self.access_tokens and self.access_tokens[self.client_region.name] and self.access_tokens[self.client_region.name]['Expires'] > datetime.now():
+            return self.access_tokens[self.client_region.name]['Token']        
         else:
             endpoint = f"/oauth/token"
 
@@ -151,9 +135,9 @@ class API():
             if data is not None:
                 expires = datetime.now() + timedelta(seconds=data['expires_in']-60)
                 
-                self.access_tokens[self.client_region] = {'Token': data['access_token'], 'Expires': expires}
+                self.access_tokens[self.client_region.name] = {'Token': data['access_token'], 'Expires': expires}
                     
-                return self.access_tokens[self.client_region]['Token']
+                return self.access_tokens[self.client_region.name]['Token']
             else:
                 raise AccessTokenException('Failed to retrieve an access token, verify your credentials & internet connectivity.')
 
@@ -204,7 +188,9 @@ class API():
 
                         supported_request_types = {
                             'GET': http_session.get,
-                            'POST': http_session.post
+                            'POST': http_session.post,
+                            'PUT': http_session.put,
+                            'DELETE': http_session.delete
                         }
 
                         if method in supported_request_types:
@@ -212,6 +198,7 @@ class API():
                                 
                                 if response.status >= 200 and response.status < 300:
                                     result = await response.json()
+                                
                                 response.raise_for_status()
                                 
                                     
@@ -220,13 +207,11 @@ class API():
                                 method, list(supported_request_types.keys())))
                 except aiohttp.ClientError as e:
                     if current_attempt == self.max_request_retries:
-                        raise
+                        if self.request_debugging:
+                            raise
+                        return result
                     current_attempt += 1
                     await asyncio.sleep(self.request_retry_delay)
-                # finally:
-                #     if result is None:
-                #         current_attempt += 1
-                #         await asyncio.sleep(self.request_retry_delay)
 
         return result
 
